> 目前为止Angular版本是11，利用TypeScript的装饰器（Decorators）特性，可以写类似Java Spring Boots面向切面风格的代码。没写过这样代码的人可能会不适应，其实基础好的话稍微看看[官方文档](https://angular.cn/docs)可以马上上手。

#### 项目介绍
本项目使用脚手架 <code>Angular CLI</code> <code>(version 11.2.12)</code> 构建。UI使用 <code>NG-ZORRO</code> ([官网](https://ng.ant.design/))，这是 Ant Design 的 Angular 版。

目前只实现纯静态登陆。**用户名:** <code>admin</code> **密码:** <code>123456</code>

截图：

**1. 默认登陆界面**

<img width="400" height="345" src="https://codechina.csdn.net/sonicwater1/ng-demo/-/raw/master/src/assets/images/img_01.png" />

<br>

**2. 点击登陆按钮验证表单非空**

<img width="400" height="388" src="https://codechina.csdn.net/sonicwater1/ng-demo/-/raw/master/src/assets/images/img_02.png" />

<br>

**3. 验证用户名密码错误**

<img width="400" height="388" src="https://codechina.csdn.net/sonicwater1/ng-demo/-/raw/master/src/assets/images/img_03.png" />

<br>

**4. 登陆成功**

<img width="400" height="388" src="https://codechina.csdn.net/sonicwater1/ng-demo/-/raw/master/src/assets/images/img_04.png" />

<br>



#### 项目目录
```powershell
ng-demo
├── README.md
├── angular.json
├── e2e
│   ├── protractor.conf.js
│   ├── src
│   │   ├── app.e2e-spec.ts
│   │   └── app.po.ts
│   └── tsconfig.json
├── karma.conf.js
├── package.json
├── src
│   ├── app
│   │   ├── app-routing.module.ts
│   │   ├── app.component.html
│   │   ├── app.component.scss
│   │   ├── app.component.spec.ts
│   │   ├── app.component.ts
│   │   ├── app.module.ts
│   │   └── components
│   │       ├── home
│   │       │   ├── home.component.html
│   │       │   ├── home.component.scss
│   │       │   └── home.component.ts
│   │       ├── login
│   │       │   ├── login.component.html
│   │       │   ├── login.component.scss
│   │       │   ├── login.component.ts
│   │       │   └── login.module.ts
│   │       └── ng-zorro-antd.module.ts
│   ├── assets
│   │   └── images
│   ├── environments
│   │   ├── environment.prod.ts
│   │   └── environment.ts
│   ├── favicon.ico
│   ├── index.html
│   ├── main.ts
│   ├── polyfills.ts
│   ├── styles.scss
│   ├── test.ts
│   └── theme.less
├── tsconfig.app.json
├── tsconfig.json
├── tsconfig.spec.json
└── tslint.json
```

#### 项目说明
<font color="#39c">/src/app/app.modules.ts</font>

```typescript
import { NgModule } from '@angular/core';
import { BrowserModule } from '@angular/platform-browser';
import { AppRoutingModule } from './app-routing.module';
import { AppComponent } from './app.component';
import { LoginModule } from './components/login/login.module';
import { NZ_I18N } from 'ng-zorro-antd/i18n';
import { zh_CN } from 'ng-zorro-antd/i18n';
import { registerLocaleData } from '@angular/common';
import zh from '@angular/common/locales/zh';
import { FormsModule, ReactiveFormsModule } from '@angular/forms';
import { HttpClientModule } from '@angular/common/http';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations'

registerLocaleData(zh);

@NgModule({
  declarations: [
    AppComponent
  ],
  imports: [
    BrowserModule,
    AppRoutingModule,
    LoginModule,
    FormsModule,
    HttpClientModule,
    BrowserAnimationsModule,
    ReactiveFormsModule
  ],
  providers: [{ provide: NZ_I18N, useValue: zh_CN }],
  bootstrap: [AppComponent]
})
export class AppModule { }
```
- 引入路由模块 <code>AppRoutingModule</code>、登陆模块 <code>LoginModule</code> 、表单模块 <code>FormsModule</code> ，并通过 <code>@NgModule</code> 装饰器的  <code>imports</code> 属性导入。
- <code>AppComponent</code> 组件放在 <code>@NgModule</code> 装饰器的  <code>declarations</code> 属性 (可声明对象表) 和  <code>bootstrap</code> 属性 (主视图，只有根模块才设置这个属性) 中。
- 关于 <code>@NgModule</code> 的使用，参考[官方文档模块部分](https://angular.cn/guide/architecture-modules) 。

<code>/src/app/app.component.html</code>
```html
<router-outlet></router-outlet>
```
只有一个路由标签。

<code>/src/app/app.routing.module.ts</code>
```typescript
import { NgModule } from '@angular/core';
import { RouterModule, Routes } from '@angular/router';

import { LoginComponent } from './components/login/login.component';
import { HomeComponent } from './components/home/home.component';

const routes: Routes = [
  {path: 'login', component: LoginComponent},
  {path: '', redirectTo: '/login', pathMatch: 'full'},
  {path: 'home', component: HomeComponent},
];

@NgModule({
  imports: [RouterModule.forRoot(routes)],
  exports: [RouterModule]
})
export class AppRoutingModule { }
```
- 设置路由的文件，项目启动后页面默认重定向到 <code>/login</code> 路由。
- 添加了 <code>/home</code> 路由，本实例暂时没用到。

<code>/src/app/components/ng-zorro-antd.module.ts</code>
```typescript
...
import { NzButtonModule } from 'ng-zorro-antd/button';
...
@NgModule({
  exports: [
    ...
    NzButtonModule,
    ...
  ]
})
export class NgZorroAntdModule {}
```

- 这个文件从 <code>ng-zorro-antd</code> 中取得所有组件 <code>Module</code> 并通过<code>@NgModule</code> 装饰器的  <code>exports</code> 属性导出。

<code>/src/app/components/login/login.moudle.ts</code>
```typescript
import { NgModule } from '@angular/core';
import { BrowserModule } from '@angular/platform-browser';
import { LoginComponent } from './login.component';
import { NgZorroAntdModule } from '../ng-zorro-antd.module';
import { FormsModule, ReactiveFormsModule } from '@angular/forms';

@NgModule({
  declarations: [
  	LoginComponent
  ],
  imports: [
    BrowserModule,
    NgZorroAntdModule,
    FormsModule,
    ReactiveFormsModule
  ],
  providers: [],
  bootstrap: [
  	LoginComponent
  ]
})
export class LoginModule { }
```
- 这里引入了刚才定义的 <code>ng-zorro-antd</code>  导出的模块 <code>NgZorroAntdModule</code>，可用在 <code>html</code> 部分，例如一个普通按钮 <code>&lt;button nz-button&gt;按钮&lt;/button&gt;</code> 标签, 增加了 <code>nz-button</code> 属性，让它在编译之后称为 <code>ng-zorro-antd</code> 按钮组件。
- 引入 <code>AppComponent</code> 组件。
- 注意引入表单相关依赖，做表单校验等会用到。

<code>/src/app/components/login/login.components.html</code> 
登陆的 <code>html</code> 部分只列出表单相关代码。
```html
...
  <form nz-form name="form" [formGroup]="validateForm" (ngSubmit)="onSubmit()">
    <nz-form-item>
      <nz-form-label [nzSpan]="4">用户名</nz-form-label>
      <nz-form-control 
        nzHasFeedback 
        [nzSpan]="20"
        nzErrorTip="请输入用户名"
      >
        <nz-input-group [nzSuffix]="suffixUsername">
          <input 
            nz-input 
            placeholder="请输入用户名" 
            name="username" 
            formControlName="username"
            required 
          />
        </nz-input-group>
        <ng-template #suffixUsername>
          <i 
            nz-icon 
            class="ant-input-clear-icon" 
            nzTheme="fill" 
            nzType="close-circle" 
            *ngIf="validateForm.controls.username.value.length>0" 
            (click)="validateForm.controls.username.setValue('')"
          ></i>
        </ng-template>

      </nz-form-control>
    </nz-form-item>
    <nz-form-item>
      <nz-form-label [nzSpan]="4">密&nbsp;&nbsp;&nbsp;&nbsp;码</nz-form-label>
      <nz-form-control 
        nzHasFeedback 
        [nzSpan]="20"
        nzErrorTip="请输入密码"
      >
        <nz-input-group [nzSuffix]="suffixPassword">
          <input 
            nz-input 
            name="password" 
            formControlName="password"
            [type]="passwordVisible ? 'text' : 'password'" 
            placeholder="请输入密码" 
            required
          />
        </nz-input-group>
        <ng-template #suffixPassword>
          <i 
            nz-icon 
            class="ant-input-clear-icon" 
            nzTheme="fill" 
            nzType="close-circle" 
            *ngIf="validateForm.controls.password.value.length>0" 
            (click)="validateForm.controls.password.setValue('')"
          ></i>
          <i 
            nz-icon 
            [nzType]="passwordVisible ? 'eye-invisible' : 'eye'" 
            (click)="passwordVisible = !passwordVisible"
          ></i>
        </ng-template>
      </nz-form-control>
    </nz-form-item>
    <nz-form-item>
      <nz-form-control [nzSpan]="20" [nzOffset]="4">
        <div nz-row class="login-form-margin">
          <div nz-col [nzSpan]="12">
            <label nz-checkbox formControlName="remember">
              <span>记住我</span>
            </label>
          </div>
          <div nz-col [nzSpan]="12" style="text-align: right;">
            <a class="login-form-forgot">忘记密码?</a>
          </div>
        </div>
      </nz-form-control>
    </nz-form-item>
    <nz-form-item>
      <nz-form-control [nzSpan]="24" [nzOffset]="0" style="text-align: center;">
        <button 
          type="submit"
          nz-button 
          style="width: 100px;" 
          nzType="primary"
          ng-disabled="form.$invalid"
        >登录</button>
      </nz-form-control>
    </nz-form-item>
  </form>
...
```

- 表单的 <code>[formGroup]="validateForm"</code> 属性绑定验证对象。
- 表单的 <code>(ngSubmit)="onSubmit()"</code> 属性绑定表单提交事件。
- 输入控件的 <code>formControlName</code> 属性绑定对应的控件名。


<code>/src/app/components/login/login.components.ts</code> 
```typescript
import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { NzMessageService } from 'ng-zorro-antd/message';

@Component({
  selector: 'app-component',
  templateUrl: './login.component.html',
  styleUrls: ['./login.component.scss']
})
export class LoginComponent implements OnInit {

  constructor(
    public login: FormBuilder,
    private message: NzMessageService
  ) {}
  
  public validateForm!: FormGroup;
  public passwordVisible:Boolean = false;
  
  // 提交表单
  onSubmit():void{

    // 更新表单未通过验证的控件高亮显示
    for (const i in this.validateForm.controls) {
      this.validateForm.controls[i].markAsDirty();
      this.validateForm.controls[i].updateValueAndValidity();
    }

    // 如果验证通过
    if(this.validateForm.status=='INVALID'){
      this.message.create('warning', '请填写用户名和密码');
      return;
    }

    // 如果验证通过
    if(this.validateForm.status=='VALID'){
      let { username, password } = this.validateForm.value;
      if( username == 'admin' && password == '123456' ){
        this.message.create('success', '登录成功');
      }else{
        this.message.create('error', '用户名密码错误');
      }
    }
  }

  // 初始化
  ngOnInit(): void {
    this.validateForm = this.login.group({
      username: ['', [Validators.required]],
      password: ['', [Validators.required]],
      remember: [true]
    });
  }
}
```

- <code>ngOnInit()</code> 是组件生命周期的一种，<code>Angular</code> 会在首次检查完组件或指令的输入属性后，紧接着调用它。这里的业务逻辑是初始化验证表单的内容。
- <code>onSubmit()</code> 提交表单的逻辑。首先验证实现了 <code>FormGroup</code> 接口的表单。高亮化未通过验证的组件（用户名和密码只做了非空验证）。然后验证是否登陆成功，做对应处理。







